#include "CubeModelClass.h"


CubeModelClass::CubeModelClass(void)
	{
	m_vertexBuffer = 0;
	m_indexBuffer = 0;

	m_model = 0;

	}

CubeModelClass::CubeModelClass(const CubeModelClass & others)
	{

	}
CubeModelClass::~CubeModelClass(void)
	{
	}

bool CubeModelClass::Initialize(ID3D11Device* device, char* modelFilename)
	{
	bool result;

	// װģ,
	result = LoadModel(modelFilename);
	if(!result)
		{
		return false;
		}

	// ģ͵ķԼ.
	CalculateModelVectors();

	// ʼ㻺Ͷ.
	result = InitializeBuffers(device);
	if(!result)
		{
		return false;
		}

	return true;
	}

void CubeModelClass::Shutdown()
	{

	// ͷŶ.
	ShutdownBuffers();
	// ͷģ.
	ReleaseModel();

	return;
	}

void CubeModelClass::Render(ID3D11DeviceContext* deviceContext)
	{
	// Ѷͼιߣ׼Ⱦ.
	RenderBuffers(deviceContext);

	return;
	}

int CubeModelClass::GetIndexCount()
	{
	//
	return m_indexCount;
	}

bool CubeModelClass::InitializeBuffers(ID3D11Device* device)
	{
	VertexType* vertices;
	unsigned long* indices;
	D3D11_BUFFER_DESC vertexBufferDesc, indexBufferDesc;
	D3D11_SUBRESOURCE_DATA vertexData, indexData;
	HRESULT result;
	int i;

	//ȣǴ2ʱŶݣԱʹá. 


	// ʱ.
	vertices = new VertexType[m_vertexCount];
	if(!vertices)
		{
		return false;
		}


	// .
	indices = new unsigned long[m_indexCount];
	if(!indices)
		{
		return false;
		}

	// Load the vertex array and index array with data.
	for(i=0; i<m_vertexCount; i++)
		{
		vertices[i].position = D3DXVECTOR3(m_model[i].x, m_model[i].y, m_model[i].z);
		vertices[i].normal = D3DXVECTOR3(m_model[i].nx, m_model[i].ny, m_model[i].nz);
		vertices[i].texture = D3DXVECTOR2(m_model[i].tu, m_model[i].tv);
		vertices[i].Kd = D3DXVECTOR4(1.0, 1.0, 1.0,1.0);
		vertices[i].Ks = D3DXVECTOR4(0.2, 0.2, 0.2,1.0);
		vertices[i].tangent = D3DXVECTOR3(m_model[i].tx, m_model[i].ty, m_model[i].tz);
		vertices[i].binormal = D3DXVECTOR3(m_model[i].bx, m_model[i].by, m_model[i].bz);
		indices[i] = i;
		}


	// ö㻺
	vertexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	vertexBufferDesc.ByteWidth = sizeof(VertexType) * m_vertexCount;
	vertexBufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	vertexBufferDesc.CPUAccessFlags = 0;
	vertexBufferDesc.MiscFlags = 0;
	vertexBufferDesc.StructureByteStride = 0;

	// ָ򱣴涥ݵʱ.
	vertexData.pSysMem = vertices;
	vertexData.SysMemPitch = 0;
	vertexData.SysMemSlicePitch = 0;

	// 㻺.
	result = device->CreateBuffer(&vertexBufferDesc, &vertexData, &m_vertexBuffer);
	if(FAILED(result))
		{
		HR(result);
		return false;
		}

	// .
	indexBufferDesc.Usage = D3D11_USAGE_DEFAULT;
	indexBufferDesc.ByteWidth = sizeof(unsigned long) * m_indexCount;
	indexBufferDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
	indexBufferDesc.CPUAccessFlags = 0;
	indexBufferDesc.MiscFlags = 0;
	indexBufferDesc.StructureByteStride = 0;

	// ָʱ.
	indexData.pSysMem = indices;
	indexData.SysMemPitch = 0;
	indexData.SysMemSlicePitch = 0;

	// .
	result = device->CreateBuffer(&indexBufferDesc, &indexData, &m_indexBuffer);
	if(FAILED(result))
		{
		HR(result);
		return false;
		}

	// ͷʱ.
	delete [] vertices;
	vertices = 0;

	delete [] indices;
	indices = 0;

	return true;
	}

void CubeModelClass::ShutdownBuffers()
	{
	// ͷŶ㻺.
	if(m_indexBuffer)
		{
		m_indexBuffer->Release();
		m_indexBuffer = 0;
		}

	// ͷ
	if(m_vertexBuffer)
		{
		m_vertexBuffer->Release();
		m_vertexBuffer = 0;
		}

	return;
	}

void CubeModelClass::RenderBuffers(ID3D11DeviceContext* deviceContext)
	{
	unsigned int stride;
	unsigned int offset;


	// ö㻺Ⱥƫ.
	stride = sizeof(VertexType); 
	offset = 0;

	//input assemberl׶ΰ󶨶㻺壬ԱܹȾ
	deviceContext->IASetVertexBuffers(0, 1, &m_vertexBuffer, &stride, &offset);

	//input assemberl׶ΰ壬ԱܹȾ
	deviceContext->IASetIndexBuffer(m_indexBuffer, DXGI_FORMAT_R32_UINT, 0);

	// Ԫ壬Ⱦб.
	deviceContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	return;
	}

bool CubeModelClass::LoadModel(char* filename)
	{
	ifstream fin;
	char input;
	int i;


	//ģļ.
	fin.open(filename);

	// ܴļ˳.
	if(fin.fail())
		{
		return false;
		}

	fin.get(input);
	while(input != ':')
		{
		fin.get(input);
		}

	//õ.
	fin >> m_vertexCount;

	// öͶһ.
	m_indexCount = m_vertexCount;

	// ģ.
	m_model = new ModelType[m_vertexCount];
	if(!m_model)
		{
		return false;
		}

	// ݿʼλ.
	fin.get(input);
	while(input != ':')
		{
		fin.get(input);
		}
	fin.get(input);
	fin.get(input);

	// ȡ
	for(i=0; i<m_vertexCount; i++)
		{
		fin >> m_model[i].x >> m_model[i].y >> m_model[i].z;
		fin >> m_model[i].tu >> m_model[i].tv;
		fin >> m_model[i].nx >> m_model[i].ny >> m_model[i].nz;
		}

	// رģļ
	fin.close();

	return true;
	}

void CubeModelClass::ReleaseModel()
	{
	if(m_model)
		{
		delete [] m_model;
		m_model = 0;
		}

	return;
	}

void CubeModelClass::CalculateModelVectors()
	{
	int faceCount, i, index;
	TempVertexType vertex1, vertex2, vertex3;
	VectorType tangent, binormal, normal;


	//.
	faceCount = m_vertexCount / 3;

	//ʼ.
	index = 0;

	// е棬򡢸Լ.
	for(i=0; i<faceCount; i++)
		{
		// Get the three vertices for this face from the model.
		vertex1.x = m_model[index].x;
		vertex1.y = m_model[index].y;
		vertex1.z = m_model[index].z;
		vertex1.tu = m_model[index].tu;
		vertex1.tv = m_model[index].tv;
		vertex1.nx = m_model[index].nx;
		vertex1.ny = m_model[index].ny;
		vertex1.nz = m_model[index].nz;
		index++;

		vertex2.x = m_model[index].x;
		vertex2.y = m_model[index].y;
		vertex2.z = m_model[index].z;
		vertex2.tu = m_model[index].tu;
		vertex2.tv = m_model[index].tv;
		vertex2.nx = m_model[index].nx;
		vertex2.ny = m_model[index].ny;
		vertex2.nz = m_model[index].nz;
		index++;

		vertex3.x = m_model[index].x;
		vertex3.y = m_model[index].y;
		vertex3.z = m_model[index].z;
		vertex3.tu = m_model[index].tu;
		vertex3.tv = m_model[index].tv;
		vertex3.nx = m_model[index].nx;
		vertex3.ny = m_model[index].ny;
		vertex3.nz = m_model[index].nz;
		index++;

		// ͸.
		CalculateTangentBinormal(vertex1, vertex2, vertex3, tangent, binormal);

		// ͸µķ.
		CalculateNormal(tangent, binormal, normal);

		// ģͽṹдµķԼ.
		m_model[index-1].nx = normal.x;
		m_model[index-1].ny = normal.y;
		m_model[index-1].nz = normal.z;
		m_model[index-1].tx = tangent.x;
		m_model[index-1].ty = tangent.y;
		m_model[index-1].tz = tangent.z;
		m_model[index-1].bx = binormal.x;
		m_model[index-1].by = binormal.y;
		m_model[index-1].bz = binormal.z;

		m_model[index-2].nx = normal.x;
		m_model[index-2].ny = normal.y;
		m_model[index-2].nz = normal.z;
		m_model[index-2].tx = tangent.x;
		m_model[index-2].ty = tangent.y;
		m_model[index-2].tz = tangent.z;
		m_model[index-2].bx = binormal.x;
		m_model[index-2].by = binormal.y;
		m_model[index-2].bz = binormal.z;

		m_model[index-3].nx = normal.x;
		m_model[index-3].ny = normal.y;
		m_model[index-3].nz = normal.z;
		m_model[index-3].tx = tangent.x;
		m_model[index-3].ty = tangent.y;
		m_model[index-3].tz = tangent.z;
		m_model[index-3].bx = binormal.x;
		m_model[index-3].by = binormal.y;
		m_model[index-3].bz = binormal.z;
		}

	return;
	}


void CubeModelClass::CalculateTangentBinormal(TempVertexType vertex1, TempVertexType vertex2, TempVertexType vertex3,
	VectorType& tangent, VectorType& binormal)
	{
	float vector1[3], vector2[3];
	float tuVector[2], tvVector[2];
	float den;
	float length;


	// 2.
	vector1[0] = vertex2.x - vertex1.x;
	vector1[1] = vertex2.y - vertex1.y;
	vector1[2] = vertex2.z - vertex1.z;

	vector2[0] = vertex3.x - vertex1.x;
	vector2[1] = vertex3.y - vertex1.y;
	vector2[2] = vertex3.z - vertex1.z;

	// tutv.
	tuVector[0] = vertex2.tu - vertex1.tu;
	tvVector[0] = vertex2.tv - vertex1.tv;

	tuVector[1] = vertex3.tu - vertex1.tu;
	tvVector[1] = vertex3.tv - vertex1.tv;


	den = 1.0f / (tuVector[0] * tvVector[1] - tuVector[1] * tvVector[0]);

	tangent.x = (tvVector[1] * vector1[0] - tvVector[0] * vector2[0]) * den;
	tangent.y = (tvVector[1] * vector1[1] - tvVector[0] * vector2[1]) * den;
	tangent.z = (tvVector[1] * vector1[2] - tvVector[0] * vector2[2]) * den;

	binormal.x = (tuVector[0] * vector2[0] - tuVector[1] * vector1[0]) * den;
	binormal.y = (tuVector[0] * vector2[1] - tuVector[1] * vector1[1]) * den;
	binormal.z = (tuVector[0] * vector2[2] - tuVector[1] * vector1[2]) * den;

	length = sqrt((tangent.x * tangent.x) + (tangent.y * tangent.y) + (tangent.z * tangent.z));

	// һ
	tangent.x = tangent.x / length;
	tangent.y = tangent.y / length;
	tangent.z = tangent.z / length;

	// ĳ.
	length = sqrt((binormal.x * binormal.x) + (binormal.y * binormal.y) + (binormal.z * binormal.z));

	// һ
	binormal.x = binormal.x / length;
	binormal.y = binormal.y / length;
	binormal.z = binormal.z / length;

	return;
	}


void CubeModelClass::CalculateNormal(VectorType tangent, VectorType binormal, VectorType& normal)
	{
	float length;


	//͸Ĳ
	normal.x = (tangent.y * binormal.z) - (tangent.z * binormal.y);
	normal.y = (tangent.z * binormal.x) - (tangent.x * binormal.z);
	normal.z = (tangent.x * binormal.y) - (tangent.y * binormal.x);

	// 㷨򳤶.
	length = sqrt((normal.x * normal.x) + (normal.y * normal.y) + (normal.z * normal.z));

	// һ.
	normal.x = normal.x / length;
	normal.y = normal.y / length;
	normal.z = normal.z / length;

	return;
	}